/*
* Copyright 2007, 2008 Ernesto Coutinho Colla
*
* "BN Parallel Package"  um nome que identifica o conjunto de programas 
* que forma uma biblioteca de rotinas que foram desenvolvidas como parte da 
* dissertao de mestrado do autor em Cincias da Computao.
*
* O conjunto de programas foi integralmente desenvolvido pelo autor e est
* disponvel sob a licena GPL (GNU General Public License). O entendimento
* integral dos termos da licena GPL  condio necessria para a utilizao
* parcial ou integral de qualquer parte deste conjunto de programas.
*
* This file is part of BN Parallel Package.
* 
* BN Parallel Package is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License.
*
* BN Parallel Package is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with BN Parallel Package. If not, see <http://www.gnu.org/licenses/>.
*/


/*
 * @file	BNLoader.c
 * @author	Ernesto Colla (ernesto@gmail.com)
 * @version	0.0.1
 * @date	Junho/2007 (2007-06-09)
 *
 * Para documentao ver BNLoader.h
 */

/* Arquivo com os cabealhos */
#include "BNLoader.h"

/* Funes */

int parse(double* values, char* string) {

	int i;
	double value;
	int nvalues = 0;
	char* buffer = NULL;
	int pt = 0;
	int done = 0;

	for (i=0;!done;i++) {

		if (string[i]!=32) {
			if (buffer==NULL) buffer = malloc(128*sizeof(char));
			buffer[pt] = string[i];
			buffer[pt+1] = '\0';
			pt++;
		}

		if ((string[i]==32 || string[i]=='\0') && pt>0) {
			values[nvalues] = atof(buffer);	
			nvalues++;
			pt = 0;
			free(buffer);	
			buffer = NULL;
		}

		if (string[i]=='\0') done = 1;

	}

	return nvalues;
}

BayesNet* bnload(char* filename) {

	// Variveis auxiliares para ler as linhas do arquivo
	char line[MAX_LINE_LENGTH];
	char tagopen[MAX_TAG_LENGTH];
	char tagclose[MAX_TAG_LENGTH];
	char tagvalue[MAX_TAGVALUE_LENGTH];

	// Handler do arquivo
	FILE* IN;

	// Varivel de controle da mquina de estado
	int state = STATE_NOT_DEFINED;
	int previous_state;
	int variables_defined = 0;

	// Variveis auxiliares para construir variveis
	char *varname;
	char **varstates;
	int varnstates;
	int vartype;
	int nvars;

	// Variveis auxiliares para construir potenciais
	Variable** potvars;
	double potvalues[MAX_POTENTIAL_VALUES];
	int potfor;
	int potgiven;
	int potnvalues;

	// Hash que ser utilizado para armazenar o id das variveis
	Hash* hashvars = newHash();
	
	// Componentes da rede Bayesiana
	BayesNet* bayesnet = newBayesNet();

	// Variveis auxiliares
	Vertex* vertex;
	VertexNode* current;
	VertexLinkedList* parents;
	Variable** variables;
	Potential** potentials;

	int i;

	// Array de nomes de estados das variveis
	varstates = malloc(1024*sizeof(char*));

	// Array auxiliar de variveis
	variables = malloc(1024*sizeof(Variable*));
	nvars = 0;

	// Array auxiliar de potenciais
	potentials = malloc(1024*sizeof(Potential*));

	IN = fopen (filename, "r");
	if (IN == NULL) {
		fprintf (stderr, "%s: %s (%d)\n", filename,strerror(errno),errno);
		exit (EXIT_FAILURE);
	}

	while(!feof(IN) && state!=STATE_FINISHED) {
		fscanf(IN,"%[^\n\r]%*\n\r",line);
		// printf("line: %s\n",line);
		tagopen[0]='\0';
		tagvalue[0]='\0';
		tagclose[0]='\0';

		if (state!=STATE_COMMENT && sscanf(line,"<%[A-Za-z]>%[^<]</%[A-Za-z]>",tagopen,tagvalue,tagclose)){
			// printf("open tag: %s\n",line);
			switch(state) {
				case STATE_NOT_DEFINED:
					// if (strstr((char*)strupr(tagopen),(char*)TAG_BIF)!=NULL) state = STATE_BIF;
					// if (strcasestr((char*)tagopen,(char*) TAG_BIF)) state = STATE_BIF;
					if (strcasestr(tagopen,TAG_BIF)) state = STATE_BIF;
				break;

				case STATE_BIF:
					// if (strcmp((char*)strupr(tagopen),(char*)TAG_NETWORK)==0) state = STATE_NETWORK;
					if (strcasecmp(tagopen,TAG_NETWORK)==0) state = STATE_NETWORK;
				break;

				case STATE_NETWORK:
					if (strcasestr(tagopen,TAG_VARIABLE)) {
						state = STATE_VARIABLE;
						varnstates = 0;
					} else if (strcasestr(tagopen,TAG_PROBABILITY) || strcasestr(tagopen,TAG_DEFINITION)) {
						state = STATE_PROBABILITY;
					} else if (strcasecmp(tagopen,TAG_NAME)==0) {
						bayesnet->graph->name = strdup(tagvalue);
					}
				break;

				case STATE_VARIABLE:
					if (strcasecmp(tagopen,TAG_NAME)==0) {
						varname = strdup(tagvalue);
						vartype = VARIABLE_TYPE_DISCRETE; // Por default a varivel  discreta
						hput(&hashvars,strdup(tagvalue),nvars);
					} else if (strcasecmp(tagopen,TAG_VALUE)==0 || strcasecmp(tagopen,TAG_OUTCOME)==0) {
						varstates[varnstates]=strdup(tagvalue);
						varnstates++;
					} else if (strcasecmp(tagopen,TAG_TYPE)==0) {
						if (strcasecmp(tagopen,TYPE_CONTINUOUS)==0) {
							vartype = VARIABLE_TYPE_CONTINUOUS;
						} else {
							vartype = VARIABLE_TYPE_DISCRETE;
						}
					}
				break;

				case STATE_PROBABILITY:
					// Acrescenta as variveis  rede
					if (!variables_defined) {
						bayesnet->variables = malloc(nvars*sizeof(Variable*));
						bayesnet->potentials = malloc(nvars*sizeof(Potential*)); // #potenciais = #variveis
						for (i=0;i<nvars;i++) {
							bayesnet->variables[i] = variables[i];
							vertex = buildVertex(i,variables[i]->name);
							addVertex(&bayesnet->graph,&vertex);
						}
						variables_defined = 1;
					}

					// Analisa as tags de PROBABILITY
					if (strcasecmp(tagopen,TAG_FOR)==0) {
						potfor = (int)hget(hashvars,tagvalue);
					} else if (strcasecmp(tagopen,TAG_GIVEN)==0) {
						potgiven = (int)hget(hashvars,tagvalue);
						addArcByIds(&bayesnet->graph,potgiven,potfor);
					} else if (strcasecmp(tagopen,(char*)TAG_TABLE)==0) {
						potnvalues = parse(potvalues,tagvalue);
					}
				break;
			}

		} else if (state!=STATE_COMMENT && sscanf(line,"</%[A-Za-z]>",tagclose)) {
			switch(state) {

				case STATE_NETWORK:
					if (strcasecmp(tagclose,TAG_NETWORK)==0) state = STATE_BIF;
				break;

				case STATE_BIF:
					if (strcasecmp(tagclose,TAG_BIF)==0) state = STATE_FINISHED;
				break;

				case STATE_VARIABLE:
					// Monta a varivel
					if (strcasecmp(tagclose,TAG_VARIABLE)==0) {
						variables[nvars] = buildVariable(nvars,varname,vartype,varnstates,varstates);
						nvars++;
						for (i=0;i<varnstates;i++) {free(varstates[i]); varstates[i]=NULL;}
						free(varname);
						varname = NULL;
						state = STATE_NETWORK;
					}
				break;

				case STATE_PROBABILITY:
					// Monta o potencial
					if (strcasestr(tagclose,TAG_PROBABILITY) || strcasestr(tagclose,TAG_DEFINITION)) {
						vertex = getVertex(&(bayesnet->graph),potfor);
						parents = getVertexParents(vertex);
						potvars = malloc((length(parents)+1)*sizeof(Variable*));

						i = length(parents);
						current = parents->first;
						while(current!=NULL) {
							potvars[i--] = bayesnet->variables[current->vertex->id];	
							current = current->next;
						}
						potvars[0] = bayesnet->variables[vertex->id];	
						bayesnet->potentials[bayesnet->variables[vertex->id]->id] = buildPotential(bayesnet->variables[vertex->id]->id,length(parents)+1,potvars,potvalues);
						free(potvars);
						potvars = NULL;
						state = STATE_NETWORK;
					}
				break;
			}
		} else {
			if (state!=STATE_COMMENT && strcasestr(line,"<!--")) {
				previous_state = state;
				state = STATE_COMMENT;
			}

			if (strcasestr(line,"-->")) {
				state = previous_state;
			}
		}
	}

	/* Fecha o arquivo */
	fclose(IN);

	// Desaloca a memria utilizada
	varname = NULL;
	potvars = NULL;
	vertex = NULL;
	current = NULL;
	parents = NULL;

	// Destri a Hash
	destroyHash(&hashvars);
	hashvars = NULL;

	// Array de nomes de estados das variveis
	free(varstates);
	varstates = NULL;

	// Array auxiliar de variveis
	free(variables);
	variables = NULL;

	// Array auxiliar de potenciais
	free(potentials);
	potentials = NULL;

	// Retorna a Rede Bayesiana
	return bayesnet;
}

int* bnloadquery(int* nxq, char* filename, BayesNet* bayesnet) {

	// Variveis auxiliares para ler as linhas do arquivo
	char line[MAX_LINE_LENGTH];
	char tagopen[MAX_TAG_LENGTH];
	char tagclose[MAX_TAG_LENGTH];
	char tagvalue[MAX_TAGVALUE_LENGTH];

	// Handler do arquivo
	FILE* IN;

	// Varivel de controle da mquina de estado
	int state = STATE_NOT_DEFINED;
	int previous_state;

	// Variveis auxiliares para construir variveis
	char **varnames;
	int nvars = 0;

	// Array de variveis questionadas
	int* xq;

	// Variveis auxiliares
	int i;

	// Hash que ser utilizado para armazenar o id das variveis
	Hash* hashvars = newHash();
	
	IN = fopen (filename, "r");
	if (IN == NULL) {
		fprintf (stderr, "%s: %s (%d)\n", filename,strerror(errno),errno);
		exit (EXIT_FAILURE);
	}

	// Aloca espao para o nome das variveis
	varnames = malloc(bnsize(bayesnet)*sizeof(char*));

	while(!feof(IN) && state!=STATE_FINISHED) {
		fscanf(IN,"%[^\n\r]%*\n\r",line);
		tagopen[0]='\0';
		tagvalue[0]='\0';
		tagclose[0]='\0';

		if (state!=STATE_COMMENT && sscanf(line,"<%[A-Za-z]>%[^<]</%[A-Za-z]>",tagopen,tagvalue,tagclose)){
			switch(state) {
				case STATE_NOT_DEFINED:
					if (strcasecmp(tagopen,TAG_QUERY)==0) state = STATE_QUERY;
				break;
				case STATE_QUERY:
					if (strcasecmp(tagopen,TAG_VARIABLE)==0) {
						varnames[nvars] = strdup(tagvalue);
						nvars++;
					}
				break;
			}
		} else if (state!=STATE_COMMENT && sscanf(line,"</%[A-Za-z]>",tagclose)) {
			switch(state) {
				case STATE_QUERY:
					// Aloca o espao para o array
					xq = malloc(nvars*sizeof(int));

					// Monta o hash
					for (i=0;i<bnsize(bayesnet);i++) hput(&hashvars,strdup(bayesnet->variables[i]->name),i);

					// Atribui o id da varivel ao array
					for (i=0;i<nvars;i++) {
						xq[i] = (int)hget(hashvars,varnames[i]);
						free(varnames[i]);
					}

					if (strcasecmp(tagclose,TAG_BIF)==0) state = STATE_FINISHED;
				break;
			}
		} else {
			if (state!=STATE_COMMENT && strcasestr(line,"<!--")) {
				previous_state = state;
				state = STATE_COMMENT;
			}

			if (strcasestr(line,"-->")) {
				state = previous_state;
			}
		}
	}

	/* Fecha o arquivo */
	fclose(IN);

	// Desaloca a memria utilizada

	// Destri a Hash
	destroyHash(&hashvars);
	hashvars = NULL;

	// Array de nomes das variveis
	free(varnames);
	varnames = NULL;

	// Retorna o vetor com as queries
	*nxq = nvars;
	return xq;
}

int* bnloadevidences(int* nxe, Potential** findings, char* filename, BayesNet* bayesnet) {

	// Variveis auxiliares para ler as linhas do arquivo
	char line[MAX_LINE_LENGTH];
	char tagopen[MAX_TAG_LENGTH];
	char tagclose[MAX_TAG_LENGTH];
	char tagvalue[MAX_TAGVALUE_LENGTH];

	// Handler do arquivo
	FILE* IN;

	// Varivel de controle da mquina de estado
	int state = STATE_NOT_DEFINED;
	int previous_state;

	// Variveis auxiliares para construir variveis e o potenciais
	Variable** vars;
	double* values;
	int varid;

	// Array auxiliar de variveis observadas
	int* aux;

	// Array de variveis observadas
	int* xe;

	// Contador auxiliar
	int ctxe;

	// Variveis auxiliares
	int i;

	// Hash que ser utilizado para armazenar o id das variveis
	Hash* hashvars = newHash();

	// Array de evidncias
	aux = malloc(bnsize(bayesnet)*sizeof(int));

	// Inicializa o contador auxiliar
	ctxe = 0;

	// Monta o hash auxiliar e a base do array de evidencias
	for (i=0;i<bnsize(bayesnet);i++) {
		hput(&hashvars,strdup(bayesnet->variables[i]->name),i);
		aux[i] = -1;
		findings[i] = NULL;
	}

	// Abre o arquivo
	IN = fopen (filename, "r");
	if (IN == NULL) {
		fprintf (stderr, "%s: %s (%d)\n", filename,strerror(errno),errno);
		exit (EXIT_FAILURE);
	}

	while(!feof(IN) && state!=STATE_FINISHED) {
		fscanf(IN,"%[^\n\r]%*\n\r",line);
		tagopen[0]='\0';
		tagvalue[0]='\0';
		tagclose[0]='\0';

		if (state!=STATE_COMMENT && sscanf(line,"<%[A-Za-z]>%[^<]</%[A-Za-z]>",tagopen,tagvalue,tagclose)){
			switch(state) {
				case STATE_NOT_DEFINED:
					if (strcasecmp(tagopen,TAG_EVIDENCES)==0) state = STATE_EVIDENCES;
				break;

				case STATE_EVIDENCES:
					if (strcasecmp(tagopen,TAG_VARIABLE)==0) {
						state = STATE_VARIABLE;
					}
				break;

				case STATE_VARIABLE:
					if (strcasecmp(tagopen,TAG_NAME)==0) {
						varid = (int)hget(hashvars,tagvalue);
					// } else if (strcmp(strupr(tagopen),TAG_VALUE)==0) {
					} else if (strcasecmp(tagopen,TAG_VALUE)==0 || strcasecmp(tagopen,TAG_OUTCOME)==0) {
						// Atualiza o potencial de finding
						if (findings[varid]==NULL) {
							// Cria um array temporrio de variveis com uma nica varivel
							vars = malloc(1*sizeof(Variable*));
							vars[0] = bayesnet->variables[varid];

							// Cria um array temporrio de valores
							values = malloc(vars[0]->nstates*sizeof(double));
							for (i=0;i<vars[0]->nstates;i++) values[i] = 0.0;

							// Cria o potencial
							findings[varid] = buildPotential (varid,1,vars,values);

							// Libera a memria alocada
							free(vars);
							free(values);
						}
					
						// Atualiza o array de findings
						for (i=0;i<bayesnet->variables[varid]->nstates;i++) {
							if (strcasecmp(bayesnet->variables[varid]->states[i],tagvalue)==0) {
								findings[varid]->values[i] = 1.0;
								break;
							}
						}

						// Marca a varivel como Xe
						aux[varid] = 1;

						// Incrementa o contador
						ctxe++;
					}
				break;
	
			}
		} else if (state!=STATE_COMMENT && sscanf(line,"</%[A-Za-z]>",tagclose)) {
			switch(state) {
				case STATE_EVIDENCES:
					if (strcasecmp(tagclose,TAG_EVIDENCES)==0) state = STATE_FINISHED;
				break;
				case STATE_VARIABLE:
					if (strcasecmp(tagopen,TAG_VARIABLE)==0) state = STATE_EVIDENCES;
				break;
			}
		} else {
			if (state!=STATE_COMMENT && strcasestr(line,"<!--")) {
				previous_state = state;
				state = STATE_COMMENT;
			}

			if (strcasestr(line,"-->")) {
				state = previous_state;
			}
		}
	}

	/* Fecha o arquivo */
	fclose(IN);

	// Desaloca a memria utilizada
	vars = NULL;
	values = NULL;

	// Destri a Hash
	destroyHash(&hashvars);
	hashvars = NULL;

	// Preprara os valores para retorno
	xe = malloc(ctxe*sizeof(int));
	*nxe = 0;
	for (i=0;i<bnsize(bayesnet);i++) {
		if (aux[i]!=-1) {
			xe[*nxe] = i;
			(*nxe)++;
			if (ctxe==*nxe) break;
		}
	}

	// Desaloca a memria do array auxiliar
	free(aux);
	aux = NULL;

	// Retorna Xe
	return xe;
}
