/*
* Copyright 2007, 2008 Ernesto Coutinho Colla
*
* "BN Parallel Package"  um nome que identifica o conjunto de programas 
* que forma uma biblioteca de rotinas que foram desenvolvidas como parte da 
* dissertao de mestrado do autor em Cincias da Computao.
*
* O conjunto de programas foi integralmente desenvolvido pelo autor e est
* disponvel sob a licena GPL (GNU General Public License). O entendimento
* integral dos termos da licena GPL  condio necessria para a utilizao
* parcial ou integral de qualquer parte deste conjunto de programas.
*
* This file is part of BN Parallel Package.
* 
* BN Parallel Package is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License.
*
* BN Parallel Package is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with BN Parallel Package. If not, see <http://www.gnu.org/licenses/>.
*/


/*
 * @author	Ernesto Colla (ernesto@gmail.com)
 * @version	0.0.1
 * @date	Julho/2007 (2007-06-07)
 *
 * Para documentao ver BNMain.h
 */

/* Lista o cabealho das funes e estrutura de dados */
#include "BNMain.h"

/* Defines */

/* Funes */

BayesNet* newBayesNet() {
	BayesNet* bayesnet = malloc(sizeof(BayesNet));
	bayesnet->graph = newGraph(DIRECTED_GRAPH);
	bayesnet->variables = NULL;
	bayesnet->potentials = NULL;

	return bayesnet;
}

BayesNet* buildBayesNet(Graph* graph, Variable** variables, Potential** potentials) {

	BayesNet* bayesnet = malloc(sizeof(BayesNet));
	// TODO: O certo seria copiar o "graph" e no apenas fazer a atribuio
	bayesnet->graph = graph;
	// TODO: O certo seria copiar o "variables" e no apenas fazer a atribuio
	bayesnet->variables = variables;
	// TODO: O certo seria copiar o "potential" e no apenas fazer a atribuio
	bayesnet->potentials = potentials;

	return bayesnet;
}

void dumpBayesNet(BayesNet* bayesnet) {
	int i;

	// Dump do grafo
	printf("Graph:");
	dumpGraph(bayesnet->graph);

	// Dump das variveis
	for (i=0;i<bnsize(bayesnet);i++) dumpVariable(bayesnet->variables[i]);

	// Dump dos potenciais
	for (i=0;i<bnsize(bayesnet);i++) dumpPotential(bayesnet->potentials[i]);
}

void destroyBayesNet(BayesNet** refBayesNet) {
	BayesNet* bayesnet = *refBayesNet;

	int i;
	int n = bnsize(bayesnet);

	// Destri o grafo
	destroyGraph(&(bayesnet->graph));

	// Destri os potenciais
	if (bayesnet->potentials!=NULL) {
		for (i=0;i<n;i++) {
			if (bayesnet->potentials[i]!=NULL) destroyPotential(&(bayesnet->potentials[i]));
		}
		free(bayesnet->potentials);
		bayesnet->potentials = NULL;
	}

	// Destri as variveis
	if (bayesnet->variables!=NULL) {
		for (i=0;i<n;i++) {
			if (bayesnet->variables[i]!=NULL) destroyVariable(&(bayesnet->variables[i]));
		}
		free(bayesnet->variables);
		bayesnet->variables = NULL;
	}

	// Libera o objeto principal
	free(bayesnet);
	bayesnet = NULL;
	refBayesNet = NULL;
}

int bnsize(BayesNet* bayesnet) {
	return sizeGraph(bayesnet->graph);
}

Potential** bnpotentials(BayesNet* bayesnet) {
	return bayesnet->potentials;
}

Variable** bnvariables(BayesNet* bayesnet) {
	return bayesnet->variables;
}
