/*
* Copyright 2007, 2008 Ernesto Coutinho Colla
*
* "BN Parallel Package"  um nome que identifica o conjunto de programas 
* que forma uma biblioteca de rotinas que foram desenvolvidas como parte da 
* dissertao de mestrado do autor em Cincias da Computao.
*
* O conjunto de programas foi integralmente desenvolvido pelo autor e est
* disponvel sob a licena GPL (GNU General Public License). O entendimento
* integral dos termos da licena GPL  condio necessria para a utilizao
* parcial ou integral de qualquer parte deste conjunto de programas.
*
* This file is part of BN Parallel Package.
* 
* BN Parallel Package is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License.
*
* BN Parallel Package is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with BN Parallel Package. If not, see <http://www.gnu.org/licenses/>.
*/


/*!
 * @file	graphtools.h
 * @author	Ernesto Colla (ernesto@gmail.com)
 * @version	0.0.1
 * @date	Fevereiro/2007
 * @brief Operaes genricas com grafos.
 *
 * Operaes bsicas implementadas:
 * 	-# Contruo do Grafo Moral (\e Moral \e Graph) de um grafo direcional.
 * 	-# Fatorao simblica do grafo moral.
 *
 */

#ifndef GRAPHTOOLS_H
#define GRAPHTOOLS_H

/* Bibliotecas especficas */
#include "graph.h"

/* Funes */

/*!
 * @brief Cria a Grafo Moral (\e Moral \e Graph) de um grafo direcionado.
 * @param[in] graph Grafo direcionado de origem.
 * @return Moral Graph do grafo de origem.
 */

Graph* buildMoralGraph (Graph* graph);

/*!
 * @brief Executa a fatorao simblica do grafo moral com base em uma ordem de eliminao predefinida.
 *
 * Os parmetros de entrada \e refEliminationTree e \e refFilledGraph devem ser explicitamente inicializados antes da chamada da funo.
 * Por questo de performance o parmetro \e refFilledGraph pode ser NULL e neste caso no ser contrudo.
 * @param[out] refEliminationTree Referncia para a rvore de eliminao quer ser preeenchida.
 * @param[out] refFilledGraph Referncia para o grafo preenchido, se NULL, no monta do grfico preenchido e ganha performance.
 * @param[in] moral	Moral graph a ser fatorado.
 * @param[in] elmorder Ordem de eliminao.
 */

void doSymbolicFactorization(Graph** refEliminationTree, Graph** refFilledGraph, Graph* moral, int* elmorder);

#endif
