/*
* Copyright 2007, 2008 Ernesto Coutinho Colla
*
* "BN Parallel Package"  um nome que identifica o conjunto de programas 
* que forma uma biblioteca de rotinas que foram desenvolvidas como parte da 
* dissertao de mestrado do autor em Cincias da Computao.
*
* O conjunto de programas foi integralmente desenvolvido pelo autor e est
* disponvel sob a licena GPL (GNU General Public License). O entendimento
* integral dos termos da licena GPL  condio necessria para a utilizao
* parcial ou integral de qualquer parte deste conjunto de programas.
*
* This file is part of BN Parallel Package.
* 
* BN Parallel Package is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License.
*
* BN Parallel Package is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with BN Parallel Package. If not, see <http://www.gnu.org/licenses/>.
*/


/*
 * @file	BNUtils.c
 * @author	Ernesto Colla (ernesto@gmail.com)
 * @version	0.0.1
 * @date	Julho/2007 (2007-06-28)
 *
 * Para documentao ver BNUtils.h
 */

/* Cabealho das funes */
#include "BNUtils.h"

/* Funes */

// Constri o Moral Graph
Graph* buildBNMoralGraph (int nxr, int* xr, BayesNet* bayesnet) {
	
	Graph* subgraph;
	Graph* moral;

	// Variveis auxiliares
	Vertex* vertex;
	VertexNode* current;
	VertexLinkedList* children;

	int* varmap;	// Mapea as variveis da rede antiga para a subrede

	int i,j;

	// Aloca o espao para o mapa de variveis
	varmap = malloc(bnsize(bayesnet)*sizeof(int));

	// Inicializa o mapa de variveis
	for (i=0;i<bnsize(bayesnet);i++) varmap[i] = 0;

	// 1. Constroi o subgrafo - mantendo os ids dos ns
	// Inicializa o subgraph
	subgraph = newGraph(DIRECTED_GRAPH);

	// Percore o vetor de variveis requireds e cria os vrtices da subrede
	for (i=0;i<nxr;i++) {
		// Atualiza o mapa de variveis indicando que o n/varivel  requirida
		varmap[xr[i]] = 1;
		// inclui o n na rede
		vertex = buildVertex(xr[i],bayesnet->variables[xr[i]]->name);
		addVertex(&subgraph,&vertex);
	}

	for (i=0;i<nxr;i++) {

		// Estabelece as ligaes
		vertex = getVertex(&bayesnet->graph,xr[i]);
		current = getVertexChildren(vertex)->first;
		while (current!=NULL) {
			if (varmap[current->vertex->id]==1) addArcByIds(&subgraph,xr[i],current->vertex->id);
			current = current->next;
		}
	}

	// 2. Gera o Moral Graph para o subgraph
	moral = buildMoralGraph(subgraph);

	// Desloca os espaos da memria;
	// Mapa de variveis
	free(varmap);
	varmap = NULL;

	// Destroy o subgraph
	destroyGraph(&subgraph);
	subgraph = NULL;

	return moral;
}

// Define a ordem de eliminao
int* defineBNEliminationOrder(Graph* moral) {
	return (int*) defineEliminationOrder(moral);
}

// Fatorao Simblica da Bayes Net
void doBNSymbolicFactorization(Graph** refElmTree, Graph** refFilledGraph, Graph* moral, int* elmorder) {
	doSymbolicFactorization(refElmTree,refFilledGraph,moral,elmorder);
}

// Determina as variveis requeridas
int* defineBNRequiredVars(int* nxr, int nxq, int* xq, int nxe, int* xe, BayesNet* bayesnet) {
	return (int*) doBayesBall(nxr,nxq,xq,nxe,xe,bayesnet);
}
