/*
* Copyright 2007, 2008 Ernesto Coutinho Colla
*
* "BN Parallel Package"  um nome que identifica o conjunto de programas 
* que forma uma biblioteca de rotinas que foram desenvolvidas como parte da 
* dissertao de mestrado do autor em Cincias da Computao.
*
* O conjunto de programas foi integralmente desenvolvido pelo autor e est
* disponvel sob a licena GPL (GNU General Public License). O entendimento
* integral dos termos da licena GPL  condio necessria para a utilizao
* parcial ou integral de qualquer parte deste conjunto de programas.
*
* This file is part of BN Parallel Package.
* 
* BN Parallel Package is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License.
*
* BN Parallel Package is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with BN Parallel Package. If not, see <http://www.gnu.org/licenses/>.
*/


/*!
 * @file	BNUtils.h
 * @author	Ernesto Colla (ernesto@gmail.com)
 * @version	0.0.1
 * @date	Julho/2007 (2007-06-28)
 * @brief 	Biblioteca do tipo "interface" (analogia com java) para encapsular implementaes de funes teis
 * em Redes Bayesianas (BN: \e Bayesian \e Network).
 *
 */

#ifndef BNUTILS_H
#define BNUTILS_H

/* Bibliotecas especficas */
#include "BNMain.h"
#include "graphtools.h"
#include "bayesball.h"
#include "threadtree.h"

/* Defines */

/* Funes */

/*!
 * @brief Constri o \e Moral \e Graph para uma Rede Bayesiana (BN: \e Bayesian \e Network) a partir de um
 * conjunto de variveis requisitadas (\e Required \e Variables).
 *
 * Ateno: No grafo moral, os \e ids das variveis requisitadas so mantidos os mesmos do grafo da rede
 * bayesiana, mesmo que no estejam em uma seqncia contnua.
 * @param[in] nxr Nmero de elementos do \e array de variveis requisitadas (\e Required \e Variables).
 * @param[in] xr \e Array de variveis requisitadas (\e Required \e Variables).
 * @param[in] bayesnet BN que ser a base para construo do \e Moral \e Graph.
 * @return \e Moral \e Graph construdo a partir das variveis requisitadas da rede bayesiana.
 */

Graph* buildBNMoralGraph (int nxr, int* xr, BayesNet* bayesnet);

/*!
 * @brief Determina a ordem de eliminao de um grafo moral de uma Rede Bayesiana.
 * @param[in] moral Grafo Moral do qual a ordem de eliminao deve ser determinada.
 * @return \e Array com os \e ids dos vrtices na ordem que devem ser eliminados.
 */

int* defineBNEliminationOrder(Graph* moral);

/*!
 * @brief Executa a fatorao simblica do grafo moral com base em uma ordem de eliminao predefinida.
 *
 * Os parmetros de entrada \e refEliminationTree e \e refFilledGraph devem ser explicitamente inicializados antes da chamada da funo.
 * Por questo de performance o parmetro \e refFilledGraph pode ser NULL e neste caso no ser contrudo.
 * @param[out] refElmTree Referncia para a rvore de eliminao quer ser preeenchida.
 * @param[out] refFilledGraph Referncia para o grafo preenchido, se NULL, no monta do grfico preenchido e ganha performance.
 * @param[in] moral	\e Moral \e Graph a ser fatorado.
 * @param[in] elmorder Ordem de eliminao.
 */

void doBNSymbolicFactorization(Graph** refElmTree, Graph** refFilledGraph, Graph* moral, int* elmorder);

/*!
 * @brief Determina o conjunto de variveis relevante para uma \e query na BN. Onde uma \e query  a inferncia 
 * sobre a BN para um determinado estado das variveis.
 * @param[out] nxr Ponteiro para um inteiro que ir armazenar o tamanho do \e array de inteiros que a funo retorna.
 * @param[in] nxq Nmero de variveis questionadas (\e Query \e Variables).
 * @param[in] xq \e Array com os \e ids das variveis questionadas (\e Query \e Variables).
 * @param[in] nxe Nmero de variveis que compe a evidncia (\e Evidence), ou seja, as variveis observadas. O estado da varivel esta
 * em outra estrutura denominada \e finding.
 * @param[in] xe \e Array com os \e ids das variveis observadas.
 * @param[in] bayesnet BN da qual se deseja obter as variveis requisitadas para a inferncia.
 * @return Um \e array de inteiros com os \e ids dos vrtices, e portanto das varaveis, requisitadas (\e Required \e Variables) para o 
 * procedimento de inferncia.
 */

int* defineBNRequiredVars(int* nxr, int nxq, int* xq, int nxe, int* xe, BayesNet* bayesnet);

#endif
