/*
* Copyright 2007, 2008 Ernesto Coutinho Colla
*
* "BN Parallel Package"  um nome que identifica o conjunto de programas 
* que forma uma biblioteca de rotinas que foram desenvolvidas como parte da 
* dissertao de mestrado do autor em Cincias da Computao.
*
* O conjunto de programas foi integralmente desenvolvido pelo autor e est
* disponvel sob a licena GPL (GNU General Public License). O entendimento
* integral dos termos da licena GPL  condio necessria para a utilizao
* parcial ou integral de qualquer parte deste conjunto de programas.
*
* This file is part of BN Parallel Package.
* 
* BN Parallel Package is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License.
*
* BN Parallel Package is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with BN Parallel Package. If not, see <http://www.gnu.org/licenses/>.
*/


/*!
 * @file	bayesball.h
 * @author	Ernesto Colla (ernesto@gmail.com)
 * @version	0.0.1
 * @date	Julho/2007 (2007-06-18)
 * @brief 	Implantao do procedimento para determinao das variveis relevantes (\e Required \e Variables) de uma Rede Bayesiana.
 * para um determinado processo de inferncia.
 *
 */

#ifndef BAYESBALL_H
#define BAYESBALL_H

/* Bibliotecas Especficas */
#include "BNMain.h"

/* Defines */

/* Typedefs */

/* Funes */

/*!
 * @brief Determina o conjunto de variveis relevante para uma \e query na BN. Onde uma \e query  a inferncia 
 * sobre a BN para um determinado estado das variveis.
 * @param[out] nxr Ponteiro para um inteiro que ir armazenar o tamanho do \e array de inteiros que a funo retorna.
 * @param[in] nxq Nmero de variveis questionadas (\e Query \e Variables).
 * @param[in] xq \e Array com os \e ids das variveis questionadas (\e Query \e Variables).
 * @param[in] nxe Nmero de variveis que compe a evidncia (\e Evidence), ou seja, as variveis observadas. O estado da varivel esta
 * em outra estrutura denominada \e finding.
 * @param[in] xe \e Array com os \e ids das variveis observadas.
 * @param[in] bayesnet BN da qual se deseja obter as variveis requisitadas para a inferncia.
 * @return Um \e array de inteiros com os \e ids dos vrtices, e portanto das varaveis, requisitadas (\e Required \e Variables) para o 
 * procedimento de inferncia.
 */

int* doBayesBall(int* nxr, int nxq, int* xq, int nxe, int* xe, BayesNet* bayesnet);

#endif
