/*
* Copyright 2007, 2008 Ernesto Coutinho Colla
*
* "BN Parallel Package"  um nome que identifica o conjunto de programas 
* que forma uma biblioteca de rotinas que foram desenvolvidas como parte da 
* dissertao de mestrado do autor em Cincias da Computao.
*
* O conjunto de programas foi integralmente desenvolvido pelo autor e est
* disponvel sob a licena GPL (GNU General Public License). O entendimento
* integral dos termos da licena GPL  condio necessria para a utilizao
* parcial ou integral de qualquer parte deste conjunto de programas.
*
* This file is part of BN Parallel Package.
* 
* BN Parallel Package is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License.
*
* BN Parallel Package is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with BN Parallel Package. If not, see <http://www.gnu.org/licenses/>.
*/


/*
 * @file	bayesball.c
 * @author	Ernesto Colla (ernesto@gmail.com)
 * @version	0.0.1
 * @date	Julho/2007 (2007-06-18)
 *
 * Para documentao ver bayesball.h
 */

/* Arquivo com os cabealhos das funes */
#include "bayesball.h"

/* Funes */

/*!
 * @brief Funo auxiliar para imprimir no \e stdout um \e array de inteiros
 * @param[in] str Nome do \e array de inteiros.
 * @param[in] n Nmero de elementos do \e array.
 * @param[in] v \e Array de inteiros a ser impresso.
 */

void printv(char* str,int n,int* v) {
	int i;
	printf("%s [",str);
	for (i=0;i<n;i++) printf("%d%s",v[i],i<n-1?", ":"");
	printf("]\n");
}

int* doBayesBall(int* nxr, int nxq, int* xq, int nxe, int* xe, BayesNet* bayesnet) {

	Vertex* vertex;
	VertexNode* current;
	VertexLinkedList* vertices;
	VertexLinkedList* parents;
	VertexLinkedList* children;

	// Vetor que indica j foi visitado a partir dos parents
	int* scheduledFromParent;
	// Vetor que indica j foi visitado a partir dos children
	int* scheduledFromChild;
	// Vetor que indica se a varivel foi observada
	int* observed;
	// Vetor de variveis requisitadas
	int* requisited;

	// Top marked
	int* top;
	// Bottom marked
	int* bottom;

	// Pilha de ns a visitar
	int* stack;
	int pt;

	// Vetor de retorno
	int* xr;
	
	// Contadores e variveis auxiliares
	int size = bnsize(bayesnet);
	int i,j;

	int k = 0;

	// Se no tem query, retorna todos os ns
	if (nxq==0) {
		*nxr = bnsize(bayesnet);
		xr = malloc(bnsize(bayesnet)*sizeof(int));
		current = bayesnet->graph->vertices->first;
		i = 0;
		while (current!=NULL) {
			xr[i++] = current->vertex->id; 
			current = current->next;	
		}

		return xr;
	}

	// Aloca os vetores auxiliares
	scheduledFromParent = malloc(size*sizeof(int));
	scheduledFromChild = malloc(size*sizeof(int));
	observed  = malloc(size*sizeof(int));
	requisited = malloc(size*sizeof(int));
	stack = malloc(size*sizeof(int));

	top = malloc(size*sizeof(int));
	bottom = malloc(size*sizeof(int));

	// Inicializa os vetores auxiliares
	for (i=0;i<size;i++) {
		scheduledFromParent[i] = 0;
		scheduledFromChild[i] = 0;
		observed[i] = 0;
		requisited[i] = 0;

		top[i] = 0;
		bottom[i] = 0;
	}

	// Mapa das variveis observadas
	for (i=0;i<nxe;i++) observed[xe[i]] = 1;

	// Todas as variveis de Xq pertencem a Xr e devem necessariamente ser visitadas
	// printv("xq:",nxq,xq);
	for (i=0;i<nxq;i++) {
		// Simula como se viesse de uma child
		scheduledFromChild[xq[i]] = 1;
		stack[i] = xq[i];
	}

	// printf("DEBUG i:%d %d\n",i,k++);

	// Atualiza o ponteiro da stack (aponta para o ndice do ltimo elemento)
	pt = i-1;

	// Inicializa o contador das variveis requisitadas	
	*nxr = 0;

	while (pt >= 0) {

		// printf("\n------------\n");
		// printf("stack pt:%d\n",pt);
		// printv("stack:",pt+1,stack);
		// printv("requisited:",size,requisited);
		// printv("top:",size,top);
		// printv("bottom:",size,bottom);

		// Retira o node da pinha
		vertex = getVertex(&(bayesnet->graph),stack[pt]);
		// Atualiza o ponteiro da pilha
		pt--;

		if (observed[vertex->id]==1 && scheduledFromChild[vertex->id]==1 && !requisited[vertex->id]) {
			requisited[vertex->id] = 1;
			(*nxr)++; 
		}

		if (observed[vertex->id]==0 && scheduledFromChild[vertex->id]==1) {
			if (top[vertex->id]==0) {
				// Marca o top
				top[vertex->id] = 1;
				if (!requisited[vertex->id]) {
					requisited[vertex->id] = 1;
					(*nxr)++; 
				}
				// Programa os pais para serem visitados
				parents = getVertexParents(vertex);
				current = parents->first;
				while (current!=NULL) {
					// printf("node: %d analisando o parent: %d\n",vertex->id,current->vertex->id);
					if (scheduledFromChild[current->vertex->id]==0) {
						scheduledFromChild[current->vertex->id] = 1;
						// printf("node: %d programando o parent: %d\n",vertex->id,current->vertex->id);
						pt++;
						stack[pt] = current->vertex->id;
						// printv("stack:",pt+1,stack);
					}
					current = current->next;
				}
			}	

			if (bottom[vertex->id]==0) {
				// Marca o bottom
				bottom[vertex->id] = 1;
				// Programa os filhos para serem visitados
				children = getVertexChildren(vertex);
				current = children->first;
				while (current!=NULL) {
					// printf("node: %d analisando a child: %d\n",vertex->id,current->vertex->id);
					if (scheduledFromParent[current->vertex->id]==0) {
						scheduledFromParent[current->vertex->id] = 1;
						// printf("node: %d programando a child: %d\n",vertex->id,current->vertex->id);
						pt++;
						stack[pt] = current->vertex->id;
					}
					current = current->next;
				}
			}
		}

		if (scheduledFromParent[vertex->id]==1) {
			if (observed[vertex->id] && top[vertex->id]==0) {
				// Marca o top
				top[vertex->id] = 1;
				if (!requisited[vertex->id]) {
					requisited[vertex->id] = 1;
					(*nxr)++; 
				}
				// Programa os pais para serem visitados
				parents = getVertexParents(vertex);
				current = parents->first;
				while (current!=NULL) {
					if (scheduledFromChild[current->vertex->id]==0) {
						scheduledFromChild[current->vertex->id] = 1;
						pt++;
						stack[pt] = current->vertex->id;
					}
					current = current->next;
				}
			}
			if (!observed[vertex->id] && bottom[vertex->id]==0) {
				// Marca o top
				bottom[vertex->id] = 1;
				// Programa os filhos para serem visitados
				children = getVertexChildren(vertex);
				current = children->first;
				while (current!=NULL) {
					// printf("node: %d analisando a child: %d\n",vertex->id,current->vertex->id);
					if (scheduledFromParent[current->vertex->id]==0) {
						scheduledFromParent[current->vertex->id] = 1;
						// printf("node: %d programando a child: %d\n",vertex->id,current->vertex->id);
						pt++;
						stack[pt] = current->vertex->id;
					}
					current = current->next;
				}
			}
		}
	}

	// printf("\nFIM---------\n");
	// printf("stack pt:%d\n",pt);
	// printv("stack:",pt+1,stack);
	// printv("requisited:",size,requisited);
	// printv("top:",size,top);
	// printv("bottom:",size,bottom);
	// printf("*nxr:%d\n",*nxr);


	// A prepara o vetor de retorno
	xr = malloc((*nxr)*sizeof(int));
	*nxr = 0;
	for (i=0;i<size;i++) {
		if (requisited[i]) {
			xr[*nxr] = i;
			(*nxr)++;
		}
	}
	
	// Desaloca os vetores auxiliares
	free(scheduledFromParent); scheduledFromParent = NULL;
	free(scheduledFromChild); scheduledFromChild = NULL;
	free(observed); observed  = NULL;
	free(requisited); requisited  = NULL;
	free(stack); stack  = NULL;
	free(top); top  = NULL;
	free(bottom); bottom  = NULL;

	return xr;
}



