/*
* Copyright 2007, 2008 Ernesto Coutinho Colla
*
* "BN Parallel Package"  um nome que identifica o conjunto de programas 
* que forma uma biblioteca de rotinas que foram desenvolvidas como parte da 
* dissertao de mestrado do autor em Cincias da Computao.
*
* O conjunto de programas foi integralmente desenvolvido pelo autor e est
* disponvel sob a licena GPL (GNU General Public License). O entendimento
* integral dos termos da licena GPL  condio necessria para a utilizao
* parcial ou integral de qualquer parte deste conjunto de programas.
*
* This file is part of BN Parallel Package.
* 
* BN Parallel Package is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License.
*
* BN Parallel Package is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with BN Parallel Package. If not, see <http://www.gnu.org/licenses/>.
*/


/*
 * @author	Ernesto Colla (ernesto@gmail.com)
 * @version	0.0.1
 * @date	Fevereiro/2007
 *
 * Para documentao ver graphtools.h
 */

/* Cabealhos das funes */
#include "graphtools.h"

/* Defines */

/* Funes */

Graph* buildMoralGraph (Graph* graph) {

	/* Declara e Inicializa o grafo moral */
	Graph* moral = newGraph(NOT_DIRECTED_GRAPH);

	/* Variveis auxiliares */
	Vertex* vertex;
	VertexNode* current;
	VertexNode* currentParent;
	VertexNode* spouses;

	int** links;	// Mapa de links para acelerar a verificao da existncia de um link
	int i,j;

	/* Aloca memria para o mapa de links */
	links = malloc(sizeGraph(graph)*sizeof(int*));

	/* Cria-se todos os vrtices do grafo */
	i = 0;
	current = graph->vertices->first;
	while (current!=NULL) {
		/* Cria a vrtice */
		vertex = buildVertex(current->vertex->id,current->vertex->name);

		/* Adiciona ao grafo */
		addVertex(&moral,&vertex);

		/* Aloca espao e inicializa o mapa de links */
		links[i] = malloc(sizeGraph(graph)*sizeof(int));
		for (j=0;j<sizeGraph(graph);j++) links[i][j] = 0;
		i = i + 1;

		/* Prximo vrtice */
		vertex = NULL; // xxx
		current = current->next;
	}

	/* Estabelece as ligaes */
	current = graph->vertices->first;
	while (current!=NULL) {

		/* Para cada vrtice liga os pais */
		currentParent = getVertexParents(current->vertex)->first;
		while (currentParent != NULL) {

			/* Estabelece o link com os parents */
			if (!links[getVertexOrder(&moral,currentParent->vertex)][getVertexOrder(&moral,current->vertex)]) {
				addLinkByIds(&moral,currentParent->vertex->id,current->vertex->id);
				links[getVertexOrder(&moral,currentParent->vertex)][getVertexOrder(&moral,current->vertex)] = 1;
			}

			/* Estabelece os moral-links */
			spouses = currentParent->next;
			while (spouses!=NULL) {
				if (!links[getVertexOrder(&moral,currentParent->vertex)][getVertexOrder(&moral,spouses->vertex)]) {
					addLinkByIds(&moral,currentParent->vertex->id,spouses->vertex->id);
					links[getVertexOrder(&moral,currentParent->vertex)][getVertexOrder(&moral,spouses->vertex)] = 1;
				}
				spouses = spouses->next;
			}

			/* Prximo vrtice da lista de parents */
			currentParent = currentParent->next;
		}

		/* Prximo vrtice do grafo */
		current = current->next;
	}
	
	/* Libera a memria do mapa de links */
	for (j=0;j<sizeGraph(graph);j++) {
		free(links[j]);
		links[j] = NULL;
	}
	free(links);
	links = NULL;

	/* Retorna o grafo moral */
	return moral;
}

void doSymbolicFactorization(Graph** refEliminationTree, Graph** refFilledGraph, Graph* moral, int* elmorder) {

		Graph* elmtree = *refEliminationTree;
		Graph* filled = *refFilledGraph;

		Vertex* a;
		Vertex* b;
		Vertex* vertex;
		Vertex* elmtreevertex;

		VertexNode* current;
		VertexNode* parent;
		VertexNode* child;
		VertexNode* node;

		int** linksByIds;	// Mapa de links para acelerar a verificao da existncia de um link
		int i,j,next;
		int maxid,size;

		/* Inicializa o grafo preenchido com o grafo moral inicial */
		current = moral->vertices->first;
		maxid = -1;
		while (current != NULL) {
			/* Obtem o maior id */
			if (maxid<current->vertex->id) maxid = current->vertex->id;
			/* Constroi o grafo preenchido */
			vertex = buildVertex(current->vertex->id,current->vertex->name);
			addVertex(&filled,&vertex);
			vertex = NULL; // xxx
			current = current->next;
		}

		/* Aloca memria para o mapa de links */
		size = maxid+1;
		linksByIds = malloc(size*sizeof(int*));

		/* Inicializa o vetor de ordens e os vrtices da rvore de eliminao */
		current = moral->vertices->first;
		i = 0;
		while (current != NULL) {
			/* Vrtice corrente */
			vertex = current->vertex;

			/* Aloca espao e inicializa o mapa de links */
			linksByIds[vertex->id] = malloc(size*sizeof(int));
			node = moral->vertices->first;
			while (node!=NULL) {
				linksByIds[vertex->id][node->vertex->id] = 0;
				node = node->next;	
			}

			child = getVertexChildren(vertex)->first;
			while (child != NULL) {

				/* Adiciona a referncia ao mapa de links */
				linksByIds[vertex->id][child->vertex->id] = 1;
				
				/* Cria o link no grafo preenchido */
				addArcByIds(&filled,vertex->id,child->vertex->id);
				child = child->next;
			}

			/* Cria o vrtice para rvore de eliminao */
			elmtreevertex = newVertex();
			elmtreevertex->name = malloc(VERTEX_NAME_MAX_LENGTH*sizeof(char));
			sprintf(elmtreevertex->name,"%d",i++);

			/* Adiciona o novo vrtice  rvore de eliminao */
			addVertex(&elmtree,&elmtreevertex);
			elmtreevertex = NULL;
			
			vertex = NULL; // xxx
			current = current->next;
		}
	
		/* Executa a fatorao */
		for (j=0;j<(sizeGraph(moral)-1);j++) {
			/* Eliminao do j-esimo n da ordem de eliminao */

			/* Determina o pai na rvore de eliminao e o vizinho mais prximo a ser eliminado */
			for (i=(j+1);i<sizeGraph(moral);i++) {
				if (linksByIds[elmorder[j]][elmorder[i]] == 1){
					a = getVertexById(&elmtree,i);
					b = getVertexById(&elmtree,j);
					addArc(&elmtree,&a,&b);
					a = NULL;
					b = NULL;
					break;
				}
			}

			/* Eliminao simplificada: preencher apenas os links que incidem sobre o vrtice vizinho MAIS prximo de ser eliminado.
			 * Conecta o n prximo de ser eliminado (next) com o n que seguinte mais prximo de ser eliminado, e que tinha 
			 * um link com o j-simo n que acabou de ser eliminado	*/

			next = i;
			for (i=next+1;i<sizeGraph(moral);i++) {
				if (linksByIds[elmorder[j]][elmorder[i]]==1 && linksByIds[elmorder[next]][elmorder[i]]==0) {
					addLinkByIds(&filled,elmorder[next],elmorder[i]);
					linksByIds[elmorder[next]][elmorder[i]] = 1;
				}
			}
		}

		/* Mapa de links libera a memria utilizada para armazenar o index de links */
		current = moral->vertices->first;
		i = 0;
		while (current != NULL) {
			for (j=i;j<current->vertex->id;j++) linksByIds[j] = NULL;
			free(linksByIds[current->vertex->id]);
			linksByIds[current->vertex->id] = NULL;
			i = current->vertex->id;
			current = current->next;
		}
		free(linksByIds);
		linksByIds = NULL;
}
