/*
* Copyright 2007, 2008 Ernesto Coutinho Colla
*
* "BN Parallel Package"  um nome que identifica o conjunto de programas 
* que forma uma biblioteca de rotinas que foram desenvolvidas como parte da 
* dissertao de mestrado do autor em Cincias da Computao.
*
* O conjunto de programas foi integralmente desenvolvido pelo autor e est
* disponvel sob a licena GPL (GNU General Public License). O entendimento
* integral dos termos da licena GPL  condio necessria para a utilizao
* parcial ou integral de qualquer parte deste conjunto de programas.
*
* This file is part of BN Parallel Package.
* 
* BN Parallel Package is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License.
*
* BN Parallel Package is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with BN Parallel Package. If not, see <http://www.gnu.org/licenses/>.
*/


/*!
 * @author	Ernesto Colla (ernesto@gmail.com)
 * @version	0.0.1
 * @date	Abril/2007
 *
 * Para documentao olhar o arquivo  hash.h
 */

/* Includes */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/* Lista o cabealho das funes e estrutura de dados */
#include "hash.h"

/* Defines */

/* Funes */

/* Calcula o index dado uma chave */

int hashIndex (char* key) {
	int hash = 0;
  	int i,j;
	int sum = 0;

	j = 0;
	i = 0;
	while (key[i]!='\0'){
		hash = hash + pow(HASH_MAX/10,j)*(key[i]);
		hash = (hash<<8)^(sum+((hash>>24)^key[i]));
		j = (j!=0 && j%hash==0)?j=0:j+1;
		sum = (sum<(HASH_MAX/2)?(sum+key[i]):key[i]);
		i++;
	}
	return hash;
}

Hash* newHash () {
	Hash* hash = malloc(sizeof(Hash));
	int i;

	// Inicializa os campos
	hash->length = 0;
	hash->nodes = malloc(HASH_MAX*sizeof(HashNode*));
	for (i=0;i<HASH_MAX;i++) hash->nodes[i] = NULL;

	return hash;
}

void destroyHash (Hash** refHash) {
	Hash* hash = *refHash;
	HashNode* current;
	HashNode* next;
	int i;

	for (i=0;i<HASH_MAX;i++) {
		if (hash->nodes[i] != NULL) {
			// Percorre a lista liberando os HashNodes
			current = hash->nodes[i];
			while (current != NULL) {
				next = current->next;
				destroyHashNode(&current);
				current = next;
			}
			hash->nodes[i] = NULL;
		}
	}
	free(hash->nodes);
	hash->nodes = NULL;

	/* Libera lista */
	free(hash);
	*refHash = NULL;
}

int hlength(Hash* hash) {
	return hash->length;
}

int hcontains (Hash* hash, char* key) {
	int index = (hashIndex(key) & (HASH_MAX-1));
	HashNode* current = hash->nodes[index];

	while(current != NULL) {
		if (strcmp(key,current->key)==0) break;
		current = current->next;		
	}

	return (current!=NULL?1:0);
}

double hget (Hash* hash, char* key) {
	int index = (hashIndex(key) & (HASH_MAX-1));
	HashNode* current = hash->nodes[index];

	while(current != NULL) {
		if (strcmp(key,current->key)==0) break;
		current = current->next;		
	}

	return (current!=NULL?current->value:'\0');
}


int hput (Hash** refHash, char* key, double value) {
	Hash* hash = *refHash;
	int index = (hashIndex(key) & (HASH_MAX-1));
	HashNode* last = NULL;
	HashNode* current = hash->nodes[index];

	// printf("inserindo: [%s] = %5.2f index=%d\n",key,value,index);

	// Percorre a lista ligada do ndice procurando pela 'key'
	while(current != NULL) {
		if (strcmp(key,current->key)==0) {
			current->value = value;
			break;
		}
		if (current->next==NULL) last = current;
		current = current->next;
	}
		
	// Se o current for NULL cri-se um novo node que passa a ser a ltimo
	if (current==NULL) {
		current = buildHashNode(key,value);
		if (last!=NULL) {
			last->next = current;
		} else {
			hash->nodes[index] = current;
		}
		hash->length++;

	}

	return hash->length;
}

HashNode* newHashNode () {
	HashNode* node = malloc(sizeof(HashNode));
	node->key = NULL;
	node->value = 0;
	node->next = NULL;
	return node;
}


HashNode* buildHashNode (char* key, double value) {
	HashNode* node = newHashNode();
	node->key = key;
	node->value = value;
	node->next = NULL;
	return node;
}

void destroyHashNode(HashNode** refNode) {
	HashNode* node = *refNode;
	free(node->key);
	node->key = NULL;
	free(node);
	*refNode = NULL;
}

void dumpHash(Hash* hash) {
	HashNode* current;
	int i = 0;
	int length;
	int collisions = 0;

	printf("length: %d\n",hash->length);
	for (i=0;i<HASH_MAX;i++) {
		current = hash->nodes[i];
		if (current!=NULL) {
			length = 0;
			while (current!=NULL) {
				length++;
				current = current->next;
			}
			if (length>1) collisions += length;
			printf("i: %d #nodes: %d\n",i,length);
		}
	}

	printf("length: %d #collisions: %d (%5.3f)\n",hash->length,collisions,100*(((double)collisions)/(double)hash->length));
}

void dumpHashContent(Hash* hash) {
	HashNode* current;
	int i = 0;
	int length;

	printf("length: %d\n",hash->length);
	for (i=0;i<HASH_MAX;i++) {
		current = hash->nodes[i];
		if (current!=NULL) {
			length = 0;
			printf("i: %d ",i);
			while (current!=NULL) {
				printf("[%s]=%5.3f ",current->key,current->value);
				length++;
				current = current->next;
			}
			printf("#nodes: %d\n",length);
		}
	}
}
